<?php

/*
 * MemcachedPlugin.class.php - configuring memcached as cache for Stud.IP
 *
 * Copyright (C) 2007 - Marcus Lunzenauer <mlunzena@uos.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 */


/**
 * @author    mlunzena
 * @copyright (c) Authors
 * @version   $Id: MemcachedPlugin.class.php 400 2007-09-07 13:55:24Z mlunzena $
 */

class MemcachedPlugin extends AbstractStudIPAdministrationPlugin {


  const CACHE_CLASS = 'MemcachedCache';
  const CACHE_FILE  = 'memcached_cache.php';
  const DOMAIN      = 'MemcachedPlugin';
  const EXTENSION   = 'Memcached';


  function __construct() {

    parent::AbstractStudIPAdministrationPlugin();

    $this->setPluginiconname("img/plugin.png");

    # navigation
    $navigation = new PluginNavigation();
    $navigation->setDisplayname(self::CACHE_CLASS . '-' .
                                dgettext(self::DOMAIN, 'Konfiguration'));
    $this->setNavigation($navigation);
  }


  function getPluginname() {
    return self::CACHE_CLASS . '-' . dgettext(self::DOMAIN, 'Konfiguration');
  }


  function setActivated($value = FALSE, $requestedbyuser = FALSE){
    parent::setActivated($value, $requestedbyuser);

    if (FALSE === $value && $this->cache_enabled()) {
      StudipCacheFactory::unconfigure();
    }
  }


  function actionShow() {

    $factory = new Flexi_TemplateFactory(dirname(__FILE__).'/templates');
    $domain = self::DOMAIN;
    $extension = self::EXTENSION;

    # is the extension available?
    if (!$this->extension_available()) {
      echo $factory->render('not_installed', compact('domain', 'extension'));
      return;
    }

    # form were submitted
    $message = array();
    if ('POST' == $_SERVER['REQUEST_METHOD']) {
      foreach (array('activate', 'deactivate') as $action) {
        if (isset($_POST[$action])) {
          $this->$action($message);
        }
      }
    }

    $cache = StudipCacheFactory::getCache();
    $current_cache_class = get_class($cache);
    $enabled = $this->cache_enabled();

    @list($hostname, $port) = StudipCacheFactory::retrieveConstructorArguments();
    $hostname = $hostname ? $hostname : "localhost";
    $port = $port ? $port : "11211";

    echo $factory->render('form', compact('message', 'enabled', 'current_cache_class', 'domain', 'extension', 'hostname', 'port'));
  }


  public function actionCheck() {
    ob_end_clean();

    $ok = $this->checkServer(@$_REQUEST['hostname'], @$_REQUEST['port']);

    $this->render_picture($ok ? "icons/16/green/accept.png" : "icons/16/red/decline.png");
    if (!$ok) {
      header('HTTP/1.1 404 Not found', TRUE, 404);
    }

    ob_start(create_function("", "return '';"));
  }


  private function activate(&$message) {
    if ($this->checkServer(@$_POST['hostname'], @$_POST['port'])) {
      StudipCacheFactory::configure(dirname(__FILE__) . '/' .
                                    self::CACHE_FILE,
                                    self::CACHE_CLASS,
                                    array(@$_POST['hostname'], @$_POST['port']));
      $message[] = array("info",
                         dgettext(self::DOMAIN, 'Cache fr Stud.IP aktiviert.'));
    }
    else {
      $message[] = array("error",
                         dgettext(self::DOMAIN, 'Cache fr Stud.IP konnte nicht aktiviert werden.'));
    }
  }


  private function deactivate(&$message) {
    StudipCacheFactory::unconfigure();
    $message[] = array("info",
                       dgettext(self::DOMAIN, 'Cache fr Stud.IP deaktiviert.'));
  }

  private function checkServer($hostname, $port) {
    $memcached = new Memcached();
    if (!$memcached->addServer($hostname, $port)) {
      return false;
    }
    $stats = $memcached->getStats();
    return isset($stats["$hostname:$port"]);
  }


  private function render_picture($filename) {
    $name = $GLOBALS['STUDIP_BASE_PATH'] . '/public/assets/images/' . $filename;
    $fp = fopen($name, 'rb');

    // send the right headers
    header("Content-Type: image/gif");
    header("Content-Length: " . filesize($name));

    // dump the picture and stop the script
    fpassthru($fp);
  }


  private function extension_available() {
    return extension_loaded("memcached");
  }


  private function cache_enabled() {
    $cache = StudipCacheFactory::getCache();
    return get_class($cache) == self::CACHE_CLASS;
  }
}

